@@ -1,6 +1,6 @@
 #!/usr/bin/env perl
 # PODNAME: export_bgc_sql_tables.pl
-# ABSTRACT: Exports SQL tables of BGC results (antiSMASH or Palantir version)
+# ABSTRACT: Exports SQL tables of BGC data (Palantir and antiSMASH annotations)
 # CONTRIBUTOR: Denis BAURAIN <denis.baurain@uliege.be>
 
 use Modern::Perl '2011';
@@ -9,11 +9,13 @@ use autodie;
 use Smart::Comments;
 
 use Carp;
+use Const::Fast;
 use Data::UUID;
 use DBI;
 use GD::Simple;
 use Getopt::Euclid qw(:vars);
 use File::Basename qw(fileparse);
+use File::ShareDir qw(dist_dir);
 use File::Temp;
 use Path::Class qw(dir file);
 use POSIX;
@@ -24,12 +26,19 @@ use Bio::MUST::Core;
 use aliased 'Bio::FastParsers::Hmmer::DomTable';
 use aliased 'Bio::MUST::Core::Taxonomy';
 use aliased 'Bio::Palantir::Parser';
-use aliased 'Bio::Palantir::Refiner';
+use aliased 'Bio::Palantir::Refiner::ClusterPlus';
+
+
+const my $DATA_PATH => dist_dir('Bio-Palantir') . '/';
 
 unless (@ARGV_infiles || $ARGV_file_table) {
     croak 'Error: use of --xml-reports or --file-table is needed';
 }
 
+if (@ARGV_types) {
+    Parser->is_cluster_type_ok(@ARGV_types);
+}
+
 # SQLite database creation
 
 # delete the previous database
@@ -46,197 +55,196 @@ my $dbh = DBI->connect($dsn,'', '', \%attr);
 
 my @stmts = <<'EOT' =~ m/(CREATE .*? \) \;)/xmsg;
 CREATE TABLE IF NOT EXISTS Clusters (
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NOT NULL,
-    type            TEXT       NOT NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    lineage         TEXT       NOT NULL,
-    FOREIGN KEY(lineage) REFERENCES lineages(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NOT NULL,
+type            TEXT       NOT NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+lineage         TEXT       NOT NULL,
+FOREIGN KEY(lineage) REFERENCES lineages(id)
 );
 CREATE TABLE IF NOT EXISTS Genes ( 
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NOT NULL,
-    name            TEXT       NOT NULL,
-    smcog           TEXT       NULL,
-    bgc_type        TEXT       NULL,
-    domains_detail  TEXT       NULL,
-    monomers        TEXT       NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    cluster         TEXT       NOT NULL,
-    FOREIGN KEY(cluster) REFERENCES clusters(id),
-    FOREIGN KEY(id)   REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NOT NULL,
+name            TEXT       NOT NULL,
+smcog           TEXT       NULL,
+bgc_type        TEXT       NULL,
+domains_detail  TEXT       NULL,
+monomers        TEXT       NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+cluster         TEXT       NOT NULL,
+FOREIGN KEY(cluster) REFERENCES clusters(id),
+FOREIGN KEY(id)   REFERENCES sequences(id)
 );
 CREATE TABLE IF NOT EXISTS Modules ( 
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NOT NULL,
-    domains         TEXT       NOT NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    cluster         TEXT       NOT NULL,
-    FOREIGN KEY(cluster) REFERENCES clusters(id),
-    FOREIGN KEY(id)   REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NOT NULL,
+domains         TEXT       NOT NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+cluster         TEXT       NOT NULL,
+FOREIGN KEY(cluster) REFERENCES clusters(id),
+FOREIGN KEY(id)   REFERENCES sequences(id)
 );
 CREATE TABLE IF NOT EXISTS Domains (
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    base_id         TEXT       NULL,
-    rank            INTEGER    NOT NULL,
-    activity        TEXT       NULL,
-    chemistry       TEXT       NULL,
-    monomer         TEXT       NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    module          TEXT       NOT NULL,
-    gene            TEXT       NOT NULL,
-    FOREIGN KEY(module) REFERENCES modules(id),
-    FOREIGN KEY(gene)   REFERENCES genes(id),
-    FOREIGN KEY(id)     REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+base_id         TEXT       NULL,
+rank            INTEGER    NOT NULL,
+activity        TEXT       NULL,
+chemistry       TEXT       NULL,
+monomer         TEXT       NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+module          TEXT       NOT NULL,
+gene            TEXT       NOT NULL,
+FOREIGN KEY(module) REFERENCES modules(id),
+FOREIGN KEY(gene)   REFERENCES genes(id),
+FOREIGN KEY(id)     REFERENCES sequences(id)
 );
 CREATE TABLE IF NOT EXISTS Genes_plus ( 
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NOT NULL,
-    name            TEXT       NOT NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    cluster         TEXT       NOT NULL,
-    FOREIGN KEY(cluster) REFERENCES clusters(id),
-    FOREIGN KEY(id)   REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NOT NULL,
+name            TEXT       NOT NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+cluster         TEXT       NOT NULL,
+FOREIGN KEY(cluster) REFERENCES clusters(id),
+FOREIGN KEY(id)   REFERENCES sequences(id)
 );
 CREATE TABLE IF NOT EXISTS Domains_plus (
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NOT NULL,
-    activity        TEXT       NULL,
-    chemistry       TEXT       NULL,
-    monomer         TEXT       NULL,
-    subtype         TEXT       NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    evalue          INTEGER    NOT NULL,
-    score           INTEGER    NOT NULL,
-    subtype_evalue  TEXT       NULL,
-    subtype_score   TEXT       NULL,
-    base_id         TEXT       NULL,
-    module          TEXT       NOT NULL,
-    gene            TEXT       NOT NULL,
-    FOREIGN KEY(module) REFERENCES modules_plus(id),
-    FOREIGN KEY(gene)   REFERENCES genes_plus(id),
-    FOREIGN KEY(id)     REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NOT NULL,
+activity        TEXT       NULL,
+chemistry       TEXT       NULL,
+monomer         TEXT       NULL,
+subtype         TEXT       NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+evalue          INTEGER    NOT NULL,
+score           INTEGER    NOT NULL,
+subtype_evalue  TEXT       NULL,
+subtype_score   TEXT       NULL,
+base_id         TEXT       NULL,
+module          TEXT       NOT NULL,
+gene            TEXT       NOT NULL,
+FOREIGN KEY(module) REFERENCES modules_plus(id),
+FOREIGN KEY(gene)   REFERENCES genes_plus(id),
+FOREIGN KEY(id)     REFERENCES sequences(id)
 );
 CREATE TABLE IF NOT EXISTS Modules_plus ( 
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NOT NULL,
-    domains         TEXT       NOT NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    cluster         TEXT       NOT NULL,
-    FOREIGN KEY(cluster) REFERENCES clusters(id),
-    FOREIGN KEY(id)   REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NOT NULL,
+domains         TEXT       NOT NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+cluster         TEXT       NOT NULL,
+FOREIGN KEY(cluster) REFERENCES clusters(id),
+FOREIGN KEY(id)   REFERENCES sequences(id)
 );
 CREATE TABLE IF NOT EXISTS Sequences (
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    sequence        TEXT       NOT NULL,
-    size            INTEGER    NOT NULL
+id              TEXT       NOT NULL    PRIMARY KEY,
+sequence        TEXT       NOT NULL,
+size            INTEGER    NOT NULL
 );
 CREATE TABLE IF NOT EXISTS Lineages (
-    id              TEXT        NOT NULL    PRIMARY KEY,
-    taxid           INTEGER     NULL,
-    lineage         TEXT        NULL,
-    superkingdom    TEXT        NULL,
-    phylum          TEXT        NULL,
-    class           TEXT        NULL,
-    ordre           TEXT        NULL,
-    family          TEXT        NULL,
-    genus           TEXT        NULL,
-    species         TEXT        NULL,
-    strain          TEXT        NULL
+id              TEXT        NOT NULL    PRIMARY KEY,
+taxid           INTEGER     NULL,
+lineage         TEXT        NULL,
+superkingdom    TEXT        NULL,
+phylum          TEXT        NULL,
+class           TEXT        NULL,
+ordre           TEXT        NULL,
+family          TEXT        NULL,
+genus           TEXT        NULL,
+species         TEXT        NULL,
+strain          TEXT        NULL
 );
 CREATE TABLE IF NOT EXISTS Domains_explus (
-    id              TEXT       NOT NULL    PRIMARY KEY,
-    rank            INTEGER    NULL,
-    activity        TEXT       NULL,
-    chemistry       TEXT       NULL,
-    monomer         TEXT       NULL,
-    subtype         TEXT       NULL,
-    size            INTEGER    NOT NULL,
-    coordinates     TEXT       NOT NULL,
-    begin           INTEGER    NOT NULL,
-    end             INTEGER    NOT NULL,
-    evalue          INTEGER    NOT NULL,
-    score           INTEGER    NOT NULL,
-    subtype_evalue  TEXT       NULL,
-    subtype_score   TEXT       NULL,
-    base_id         TEXT       NULL,
-    gene            TEXT       NOT NULL,
-    FOREIGN KEY(gene)   REFERENCES genes_plus(id),
-    FOREIGN KEY(id)     REFERENCES sequences(id)
+id              TEXT       NOT NULL    PRIMARY KEY,
+rank            INTEGER    NULL,
+activity        TEXT       NULL,
+chemistry       TEXT       NULL,
+monomer         TEXT       NULL,
+subtype         TEXT       NULL,
+size            INTEGER    NOT NULL,
+coordinates     TEXT       NOT NULL,
+begin           INTEGER    NOT NULL,
+end             INTEGER    NOT NULL,
+evalue          INTEGER    NOT NULL,
+score           INTEGER    NOT NULL,
+subtype_evalue  TEXT       NULL,
+subtype_score   TEXT       NULL,
+base_id         TEXT       NULL,
+gene            TEXT       NOT NULL,
+FOREIGN KEY(gene)   REFERENCES genes_plus(id),
+FOREIGN KEY(id)     REFERENCES sequences(id)
 );
 EOT
 
 if ($ARGV_quast) {
     push @stmts, <<'EOT', 
 CREATE TABLE IF NOT EXISTS Assemblies (
-    id                     TEXT     NOT NULL    PRIMARY KEY,
-    n_contigs_gt0          INTEGER  NOT NULL,
-    n_contigs_gt1000       INTEGER  NOT NULL,
-    tot_length_gt0         INTEGER  NOT NULL,
-    tot_length_gt1000      INTEGER  NOT NULL,
-    n_contigs              INTEGER  NOT NULL,
-    largest_contig         INTEGER  NOT NULL,
-    tot_length             INTEGER  NOT NULL,
-    GC_content             INTEGER  NOT NULL,
-    N50                    INTEGER  NOT NULL,
-    N75                    INTEGER  NOT NULL,
-    L50                    INTEGER  NOT NULL,
-    L75                    INTEGER  NOT NULL,
-    Ns_per_100kbp          INTEGER  NOT NULL,
-    br_n_contigs_gt0       INTEGER  NULL,
-    br_n_contigs_gt1000    INTEGER  NULL,
-    br_tot_length_gt0      INTEGER  NULL,
-    br_tot_length_gt1000   INTEGER  NULL,
-    br_n_contigs           INTEGER  NULL,
-    br_largest_contig      INTEGER  NULL,
-    br_tot_length          INTEGER  NULL,
-    br_GC_content          INTEGER  NULL,
-    br_N50                 INTEGER  NULL,
-    br_N75                 INTEGER  NULL,
-    br_L50                 INTEGER  NULL,
-    br_L75                 INTEGER  NULL,
-    br_Ns_per_100kbp       INTEGER  NULL,
-    FOREIGN KEY (id) REFERENCES clusters(lineage)
+id                     TEXT     NOT NULL    PRIMARY KEY,
+n_contigs_gt0          INTEGER  NOT NULL,
+n_contigs_gt1000       INTEGER  NOT NULL,
+tot_length_gt0         INTEGER  NOT NULL,
+tot_length_gt1000      INTEGER  NOT NULL,
+n_contigs              INTEGER  NOT NULL,
+largest_contig         INTEGER  NOT NULL,
+tot_length             INTEGER  NOT NULL,
+GC_content             INTEGER  NOT NULL,
+N50                    INTEGER  NOT NULL,
+N75                    INTEGER  NOT NULL,
+L50                    INTEGER  NOT NULL,
+L75                    INTEGER  NOT NULL,
+Ns_per_100kbp          INTEGER  NOT NULL,
+br_n_contigs_gt0       INTEGER  NULL,
+br_n_contigs_gt1000    INTEGER  NULL,
+br_tot_length_gt0      INTEGER  NULL,
+br_tot_length_gt1000   INTEGER  NULL,
+br_n_contigs           INTEGER  NULL,
+br_largest_contig      INTEGER  NULL,
+br_tot_length          INTEGER  NULL,
+br_GC_content          INTEGER  NULL,
+br_N50                 INTEGER  NULL,
+br_N75                 INTEGER  NULL,
+br_L50                 INTEGER  NULL,
+br_L75                 INTEGER  NULL,
+br_Ns_per_100kbp       INTEGER  NULL,
+FOREIGN KEY (id) REFERENCES clusters(lineage)
 );
 EOT
 }
 
 # initialize the database with table statements
 for my $stmt (@stmts) {
-$dbh->do($stmt);
+    $dbh->do($stmt);
 }
 
 # fill the SQL tables
-
 my %table_for;
 
 # optionally use an idm file to connect genomes/proteomes and antiSMASH report filenames
 my %accession_for;
 if ($ARGV_idm_file) {
-open my $in, '<', $ARGV_idm_file;
+    open my $in, '<', $ARGV_idm_file;
 
-my @idm_ext = qw(.faa .fna .fa);
+    my @idm_ext = qw(.faa .fna .fa);
 
     while (my $line = <$in>) {
         
@@ -274,89 +282,26 @@ for my $infile ($ARGV_file_table ? @infiles : @ARGV_infiles) {
     my @xml_ext = qw(_ensembl.xml _ncbi.xml _lm.xml .xml);
     my ($base, $dir, $suffix) = fileparse($infile, @xml_ext);
 
-    my $defline = $base;
+    # generate Lineage table and get lineage_id
+    my $lineage_id = get_lineageid_and_fill_lineage_table($base);
 
-    # Lineages table
-    my ($lineage_id, @lineage_elmts);
-    if ($ARGV_taxdir) {
-    
-        # get taxonomic informations
-        my $tax = Taxonomy->new(
-                tax_dir   => $ARGV_taxdir,
-                save_mem  => 1,
-                fuzzy_map => 1,
-        );
-
-        my ($gca) = $defline =~ m/ (GC[AF] \_ \d{9} \. \d) /xmsi;
-        $gca = $defline =~ m/taxid(\d+)/xmsi 
-            unless $gca;
-         
-        if ($ARGV_idm_file && ! defined $gca) {
-                $gca = $accession_for{$base};
-        }
-        
-        my ($org) = $defline =~ m/ ( [^\_\.]* \_  [^\_\.]*) /xms;
-
-        my ($taxid, @lineages);
-        if ($gca) {
-            
-            @lineages =  $tax->get_taxonomy($gca);
-            
-            $org = $lineages[-1] =~ tr/ /_/r;
-            $taxid = $tax->get_taxid_from_seq_id( $org . '@1');
-            
-            push @lineage_elmts,
-                $taxid, (join ',', @lineages), 
-                (map { $lineages[$_] } (1, 2, 3)),
-                (map { $tax->get_term_at_level($taxid, $_) }
-                    qw(order family genus species subspecies)),
-            ;
-
-            $lineage_id = $gca;
-        }
-
-        elsif ($org) {
-
-            $taxid = $tax->get_taxid_from_seq_id($org . '@1');
-
-            if ($taxid) {
-
-                @lineages =  $tax->get_taxonomy($taxid);
-                
-                push @lineage_elmts,
-                    $taxid, (join ',', @lineages),
-                    (map { $lineages[$_] } (1, 2, 3)),
-                    (map { $tax->get_term_at_level($taxid, $_) } 
-                        qw(order family genus species subspecies)),
-                ;
-            }
-
-            $lineage_id = $defline;
-        }    
-         
-
-        else {
-            $lineage_id = $defline;
-        }
-    }
-
-    else {
-        $lineage_id = $defline;
-    }
-    
-    unshift @lineage_elmts, $lineage_id;
-    
-    if (@lineage_elmts < 10) {
-        push @lineage_elmts, (('NULL') x 10);
-    }
-
-    push @{ $table_for{'Lineages'} }, join "\t", @lineage_elmts;
-
-    # generate BGC tables
 
     # load antiSMASH report
-    fill_bgc_tables($infile, 'antismash', $lineage_id, \%table_for);
-    fill_bgc_tables($infile, 'palantir' , $lineage_id, \%table_for);
+    my $parser = Parser->new( file => $infile);
+    my $root = $parser->root;
+
+    my @clusters = $root->all_clusters;
+    my @clusters_plus = map { 
+        ClusterPlus->new( 
+            _cluster => $_,
+            gap_filling => $ARGV_gap_filling,
+            undef_cleaning => $ARGV_undef_cleaning,
+        )
+    } @clusters;
+
+    # generate BGC tables
+    fill_bgc_tables('antismash', $lineage_id, \%table_for, \@clusters);
+    fill_bgc_tables('palantir' , $lineage_id, \%table_for, \@clusters_plus);
  
     if ($ARGV_quast) {
 
@@ -401,52 +346,132 @@ $dbh->disconnect;
 say 'all is good!';
 
 
-sub fill_bgc_tables {
+sub get_lineageid_and_fill_lineage_table {
 
-    my ($report_file, $annotation, $lid, $table_ref) = @_;
+    my $defline = shift;
 
-    my ($suffix, @clusters);
-    if ($annotation eq 'antismash') {
+    # Lineages table
+    my ($lineage_id, @lineage_elmts);
+    if ($ARGV_taxdir) {
+    
+        # get taxonomic informations
+        my $tax = Taxonomy->new(
+                tax_dir   => $ARGV_taxdir,
+                save_mem  => 1,
+                fuzzy_map => 1,
+        );
 
-        my $parser = Parser->new( file => $report_file );
-        my $root = $parser->root;
-        push @clusters, $root->all_clusters;
+        my ($gca) = $defline =~ m/ (GC[AF] \_ \d{9} \. \d) /xmsi;
+        $gca = $defline =~ m/taxid(\d+)/xmsi 
+            unless $gca;
+         
+        if ($ARGV_idm_file && ! defined $gca) {
+                $gca = $accession_for{$defline};
+        }
+        
+        my ($org) = $defline =~ m/ ( [^\_\.]* \_  [^\_\.]*) /xms;
 
-        $suffix = '';
+        my ($taxid, @lineages);
+        if ($gca) {
+            
+            @lineages =  $tax->get_taxonomy($gca);
+            
+            $org = $lineages[-1] =~ tr/ /_/r;
+            $taxid = $tax->get_taxid_from_seq_id( $org . '@1') // 'na';
+            
+            push @lineage_elmts,
+                $taxid, (join ',', @lineages), 
+                (map { $lineages[$_] } (1, 2, 3)),
+                (map { $tax->get_term_at_level($gca, $_) }
+                    qw(order family genus species)),
+                    $lineage_elmts[-1],
+            ;
+
+            $lineage_id = $gca;
+        }
+
+        elsif ($org) {
+
+            $taxid = $tax->get_taxid_from_seq_id($org . '@1');
+
+            if ($taxid) {
+
+                @lineages =  $tax->get_taxonomy($taxid);
+                
+                push @lineage_elmts,
+                    $taxid, (join ',', @lineages),
+                    (map { $lineages[$_] } (1, 2, 3)),
+                    (map { $tax->get_term_at_level($taxid, $_) } 
+                        qw(order family genus species)),
+                        $lineage_elmts[-1],
+                ;
+            }
+
+            $lineage_id = $defline;
+        }    
+         
+
+        else {
+            $lineage_id = $defline;
+        }
     }
 
     else {
-
-        my $refiner = Refiner->new( 
-            file => $report_file,
-            gap_filling => $ARGV_gap_filling, 
-            undef_cleaning => $ARGV_undef_cleaning,
-        );
-        push @clusters, $refiner->all_clusters;
-
-        $suffix = '_plus';
+        $lineage_id = $defline;
     }
+    
+    unshift @lineage_elmts, $lineage_id;
+    
+    if (@lineage_elmts < 10) {
+        push @lineage_elmts, (('NULL') x 10);
+    }
+
+    push @{ $table_for{'Lineages'} }, join "\t", @lineage_elmts;
+
+    return ($lineage_id);
+}
+
+sub fill_bgc_tables {
+
+    my ($annotation, $lid, $table_ref, $ref_clusters) = @_;
+
+    my $suffix = $annotation eq 'palantir'
+        ? '_plus'
+        : ''
+    ;
 
     my $ug = Data::UUID->new;
 
     CLUSTER:
-    for my $cluster (@clusters) {
+    for my $cluster (@$ref_clusters) {
 
         #TODO make an option to allow others? -> need to create new uui for genes and maybe domains that are used several times
         #TODO use the same loop (with fix&proceed for additional information) for Parser and Refiner
-        next CLUSTER 
-            if $cluster->type eq 'other';
+        if (@ARGV_types) {
+            next CLUSTER unless
+                grep { $cluster->type =~ m/$_/xmsi } @ARGV_types;
+        }
 
-        my @cluster_elmts = (
-            $cluster->uui, $cluster->rank, $cluster->type, 
-            $cluster->genomic_prot_size, 
-            (join '-', @{ $cluster->genomic_prot_coordinates }), 
-            $cluster->genomic_prot_begin, $cluster->genomic_prot_end,
-            $lid,
-        );
-        
-        @cluster_elmts = map { $_ // 'na' } @cluster_elmts;
-        push @{ $table_ref->{'Clusters'} }, join "\t", @cluster_elmts;
+        # keep root uui for clusters and clusters_plus
+        my $cluster_uui =
+            $cluster->meta->name eq 'Bio::Palantir::Refiner::ClusterPlus'
+            ? $cluster->_cluster->uui
+            : $cluster->uui
+        ;
+
+        # to not duplicate Cluster data in the SQL table
+        if ($annotation eq 'antismash') {
+            my @cluster_elmts = (
+                $cluster_uui, $cluster->rank, $cluster->type, 
+                $cluster->genomic_prot_size, 
+                (join '-', @{ $cluster->genomic_prot_coordinates }), 
+                $cluster->genomic_prot_begin, $cluster->genomic_prot_end,
+                $lid,
+            );
+            
+            @cluster_elmts = map { $_ // 'na' } @cluster_elmts;
+            push @{ $table_ref->{'Clusters'} }, join "\t", @cluster_elmts;
+        }
 
         # treat genes containing modules and note them to no repeat action
         my %module_uui_for;
@@ -462,7 +487,7 @@ sub fill_bgc_tables {
                     $module->size, 
                     (join '-', @{ $module->genomic_prot_coordinates }),
                     $module->genomic_prot_begin, $module->genomic_prot_end, 
-                    $cluster->uui,
+                    $cluster_uui,
                 );
 
                 @module_elmts = map { $_ // 'na' } @module_elmts;
@@ -497,7 +522,7 @@ sub fill_bgc_tables {
                     (join '; ', $gene->monomers), $gene->genomic_prot_size, 
                     (join '-', @{ $gene->genomic_prot_coordinates }),
                     $gene->genomic_prot_begin, $gene->genomic_prot_end, 
-                    $cluster->uui,
+                    $cluster_uui,
                 );
             }
 
@@ -507,7 +532,7 @@ sub fill_bgc_tables {
                     $gene->genomic_prot_size, 
                     (join '-', @{ $gene->genomic_prot_coordinates }), 
                     $gene->genomic_prot_begin, $gene->genomic_prot_end, 
-                    $cluster->uui,
+                    $cluster_uui,
                 );
             }    
             
@@ -589,39 +614,37 @@ sub fill_bgc_tables {
 
 sub get_function {
     my $seq = shift;
-    my $hmmdb = '/media/vol2/home/lmeunier/tools/bio-palantir/'
-        . 'data/generic_domains.hmm';
+    my $hmmdb = $DATA_PATH . 'generic_domains.hmm';
 
     my $tbout = do_hmmscan($seq, $hmmdb);
 
-    # parsing of domtblout hmmscan report 
-    my $hmmscan_report = DomTable->new( file => $tbout->filename );
+    #parsing of  domtblout hmmscan report 
+    my $report = DomTable->new( file => $tbout->filename );
 
     my %hit_for;
-    my $evalue_threshold = 10e-5;
-    my $i2;
+    my $evalue_threshold = 10e-3;
+    my $hit_i = 1;
 
     HIT:
-    while (my $hit = $hmmscan_report->next_hit) {   #TODO the method risks to be changed /BUG!!!\ 
-        
-        $i2++;
-        next HIT
-            if $hit->evalue > $evalue_threshold;
+    while (my $hit = $report->next_hit) {
 
-        $hit_for{'hit' . $i2} = {
-            map { $_ => $hit->$_ } 
-            qw(query_name target_name ali_from ali_to hmm_from hmm_to tlen qlen)
+        next HIT if $hit->evalue > $evalue_threshold;
+
+        $hit_for{'hit' . $hit_i} = { map { $_ => $hit->$_ } 
+            qw(query_name target_name ali_from ali_to hmm_from 
+                hmm_to tlen qlen) 
         };
-        
-        $hit_for{'hit' . $i2}{score}  = $hit->dom_score;
-        $hit_for{'hit' . $i2}{evalue} = $hit->i_evalue;     # i-value = independent evalue ('evalue' return cumulative evalues for the sequence, and c-evalue may be deleted because potentially e-value)
+
+        $hit_for{'hit' . $hit_i}{score}  = $hit->dom_score;
+        $hit_for{'hit' . $hit_i}{evalue} = $hit->i_evalue;     # i-value = independent evalue ('evalue' return cumulative evalues for the sequence, and c-evalue may be deleted because potentially e-value)
+        $hit_i++;
     }
 
     my $best_hit = (sort { $hit_for{$b}{score} <=> $hit_for{$a}{score} } 
-            keys %hit_for)[0];
+        keys %hit_for)[0];
 
     my $function = $hit_for{$best_hit}{target_name};
-    
+
     my $activity = 
     $function =~ m/^A$ | AMP-binding | A-OX/xms ? 'A' : 
     $function =~ m/^PCP | PP-binding/xms ? 'PCP' : 
@@ -639,91 +662,42 @@ sub get_function {
 
 sub do_hmmscan {
     my ($seq, $hmmdb) = @_; 
-   
-    my $template = 'tempfile_XXXXXXXXXXXXXXXX';
-    my $query = File::Temp->new($template, suffix => '.faa', unlock => 1); 
+
+    my $template = 'tempfile_XXXXXXXXXX';
+    my $query = File::Temp->new($template, suffix => '.faa'); 
     print $query '>query' . "\n" . $seq;
 
     my $pgm = 'hmmscan';
 
-    my $ARGV_cpu = 1;
-    my $tbout = File::Temp->new($template, suffix => '_domtblout.tsv'); #, unlock => $ARGV_mode_debug ? 0 : 1);  
+    my $tbout = File::Temp->new($template, suffix => '_domtblout.tsv');
     my $opt = ' --domtblout ' . $tbout . ' --cpu ' . $ARGV_cpu;
-    my $log = File::Temp->new($template, suffix => '_hmmscan.log', unlock => 1);
 
-    my $cmd = "$pgm $opt $hmmdb $query > $log"; 
+    my $cmd = "$pgm $opt $hmmdb $query"; 
     system $cmd;
-   
+
     return $tbout;
 }
 
-
-# resistance gene search
-
-# $hmmdb = '/media/vol2/home/lmeunier/data/resistance_genes/Resfams.hmm'; # use full version?
-# $query = $proteome; 
-# $pgm = 'hmmscan';
-# 
-# $tbout 
-#   = File::Temp->new(
-#       $template, 
-#       suffix => '_domtblout.tsv', 
-#       unlock => $ARGV_mode_debug ? 0 : 1
-#     )
-# ;  
-# $opt = '--domtblout ' . $tbout . ' --cpu ' . $ARGV_cpu;
-# 
-# $cmd = "$pgm $opt $hmmdb $query"; 
-# system $cmd;
-# 
-# # parse domtblout hmmer report 
-# $report = DomTable->new( file => $tbout->filename );
-# 
-# my %rg_for; # resistance gene
-# $i;
-# 
-# HIT:
-# while (my $hit = $report->next_hit) {
-#     
-#     if ($ARGV{'--evalue-threshold'}) {
-#         next HIT if $hit->evalue > $ARGV{'--evalue-threshold'};
-#     }
-#     
-#     $i++;
-# 
-#     $rg_for{ $hit->query_name }{ 'hit_' . $i } = {
-#         map { $_ => $hit->$_ } 
-#         qw(target_name ali_from ali_to hmm_from hmm_to tlen score evalue )
-#     };
-# 
-# }
-# 
-# ### %rg_for
-
 __END__
 
 =pod
 
 =head1 NAME
 
-export_bgc_sql_tables.pl - Exports SQL tables of BGC results (antiSMASH or Palantir version)
+export_bgc_sql_tables.pl - Exports SQL tables of BGC data (Palantir and antiSMASH annotations)
 
 =head1 VERSION
 
-version 0.191620
+version 0.191800
 
 =head1 NAME
 
-export_bgc_sql_tables.pl - This tool creates a sql database for the AntiSMASH
-results and allow to make queries on it.
-
-=head1 VERSION
-
-This documentation refers to  version 0.0.1
+export_bgc_sql_tables.pl - This tool exports SQL tables structuring the BGC data
+from antiSMASH reports and annotated with Palantir.
 
 =head1 USAGE
 
-    $0 [options] --paths <biosynml_path> --taxdir <dir> 	
+    $0 [options] --infiles [=] <report_paths>.../--file-table [=] <report.list>
 
 =head1 REQUIRED ARGUMENTS
 
@@ -731,31 +705,44 @@ This documentation refers to  version 0.0.1
 
 =back
 
-=head1 OPTIONS
+=head1 OPTIONAL ARGUMENTS
 
 =over
 
-=item --infiles [=] <paths>...
+=item --infiles [=] <report_paths>...
 
-Path to biosynML.xml or regions.js  files, including at least the directory
-containing this file. This option can takes multiple values.
+Paths to biosynML.xml (antiSMASH 3-4) or regions.js (antiSMASH 5) files.
+This option can takes multiple values.
 
 =item --file-table [=] <tsv_file>
 
-TSV (Tabulation separated values) format file to give non ambiguously the path
+TSV (Tab-Separated Values) format file to give non ambiguously the path
 of xml reports, proteomes and quast files. Order : xml reports (1st column),
 proteomes (2nd column) and quast files (3rd column). If you only want to parse
 xml and quast reports, you can follow this format : "biosynML.xml    undef
 quast.tsv".
 
+=item --types [=] <str>...
+
+Filter clusters on a/several specific type(s). 
+
+Types allowed: acyl_amino_acids, amglyccycl, arylpolyene, bacteriocin, 
+butyrolactone, cyanobactin, ectoine, hserlactone, indole, ladderane, 
+lantipeptide, lassopeptide, microviridin, nrps, nucleoside, oligosaccharide, 
+otherks, phenazine, phosphonate, proteusin, PUFA, resorcinol, siderophore, 
+t1pks, t2pks, t3pks, terpene.
+
+Any combination of these types, such as nrps-t1pks or t1pks-nrps, is also
+allowed. The argument is repeatable.
+
 =item --taxdir [=] <dir>
 
 Path to a local mirror of the NCBI Taxonomy database.
 
 =item --idm[-file] [=] <file>
 
-Path to an id mapper file to retrieve the assemblies accession numbers. The
-file should be in tabular format with accession number in the secund column.
+Path to an id mapper file to retrieve the assembly accession numbers. The
+file should be in tabular format with accession numbers in the second column.
 
 =item --proteomes
 
@@ -766,58 +753,56 @@ database.
 
 Create an additionnal table "Assemblies" with Quast statistics. For this
 option, you need to use the transposed_report.tsv output of quast and name it
-with the basename of your xml file.  For example, if you use my_org.xml, name
+with the basename of your report file. For example, if you use my_org.xml, name
 your Quast file my_org.tsv.
 
 =item --new-db
 
-Remove the previous sql tables to start over the db
+Remove the previous sql tables to start over the db.
 
-=item --db-name [=] <name>
+=item --db-name [=] <str>
 
-Name of your database [default: bgc-db]
+Name of your database [default: bgc_db].
 
-=for Euclid: name.default: 'bgc_db'
+=for Euclid: str.type: str
+    str.default: 'bgc_db'
 
 =item --gap-filling [=] <bool>
 
-Tries to find domains if gaps present in clusters.
+Tries to find domains if gaps present in clusters [default: 1].
 
 =for Euclid: bool.type: num 
     bool.default: 1
 
 =item --undef-cleaning [=] <bool>
 
-Eliminates undef domains from antiSMASH output that can't be recovered.
+Eliminates undef domains from antiSMASH output that can't be recovered
+[default: 1].
 
 =for Euclid: bool.type: num 
     bool.default: 1
 
 =item --undef-recov [=] <bool>
 
-Try to recover antismash undef domain values.
+Try to recover antismash undef domain values [default: 0].
 
 =for Euclid: bool.type: num 
     bool.default: 0
 
 =item --evalue-threshold [=] <n>
 
+E-value threshold to apply in HMMER searches [default: 1e-4].
+
 =for Euclid: n.type: num 
     n.default: 1e-4
 
-=item --mode-debug
-
-Conserve the temporary files.
-
 =item --cpu [=] <n>
 
-Number of threads/cpus to use. 
+Number of threads/cpus to use [default: 1].
 
 =for Euclid: n.type: int 
     n.default: 1 
 
-=item --more
-
 =item --version
 
 =item --usage
